/* File: 1000 HwrBacklightPat.c
   ============================
   This file contains a patch for the trap "HwrBacklight".  All it does it invert
   the pixel polarity and then pass control over to the standard HwrBacklight function.
   
   Written by Douglas I. Anderson (dianders@cs.stanford.edu)

   Hack patch info
   ---------------
   code id:   1000
   atrap:	  A2EA (sysTrapHwrBacklight)

   History
   -------
   03/23/99 Created
   03/23/99 Added always/usually/sometimes modes.
   06/06/99 Changed so that turning off the backlight always turns invert off.
   01/04/02 Removed sometimes / usually.
 */

// ----------------------------------
// Interfaces...
// ----------------------------------

// Silly precompiled harders aren't right on my system--don't really
// need them anyway...
#define PILOT_PRECOMPILED_HEADERS_OFF


/* System-wide */
#include <Pilot.h>
#include <Graffiti.h>

/* Hack specific */
#include "GreenLightHack.h"

// ----------------------------------
// Local types...
// ----------------------------------

typedef Boolean (*HwrBacklightFn) (Boolean set, Boolean newState);

// ----------------------------------
// Local function prototypes...
// ----------------------------------

// ----------------------------------
// Trap Handler...
// ----------------------------------

/* Function: MyHwrBacklight()
   --------------------------
   This is the entry function of this patch.  All system calls to HwrBacklight
   will be redirected here.  
   
   We set the invert bit in the LCD Controller if they're turning the backlight on,
   we clear it if they're turning the backlight off.
   
   @param set      Tells whether to set the backlight or just return the current
   				   value.
   @param newState Tells what value to set the backlight to.
   @return         The old value of the backlight.
 */

Boolean	MyHwrBacklight (Boolean set, Boolean newState);
Boolean	MyHwrBacklight (Boolean set, Boolean newState)
{
	HwrBacklightFn oldTrap;

	FtrGet (kCreatorCode, kHwrBacklightPatchID, (DWord*) &oldTrap);

	if (set) {
		const char kPixPolMask = 0x01;
		char * const LPOLCF = (char *)0xFFFFFA21;

		if (newState) {
			// Enable if turning on...
			(*LPOLCF) |= kPixPolMask;
		} else {
			// Disable if turning off...
			(*LPOLCF) &= ~kPixPolMask;
		}
	}
	return oldTrap (set, newState);
}











