/* File: 1000 HwrDisplayAttr.c
   ===========================
   Patches HwrDisplayAttributes so that it returns a 0 for opcode 0x0A any place
   where it was going to return a 2.  Opcode 0x0A (at least in current versions of
   the OS) is the VRAM size, which quite obviously shouldn't be 2 (0 should be returned
   on devices with no VRAM).
   
   Note: HwrDisplayAttributes is not a public function and thus is subject to change.
   This hack is intended to fix a specific bug in the Sony Clie and attempts to fix it
   only in known environments.
   
   
   Written 2001, Douglas I. Anderson (dianders@byteswapped.com)

   Hack patch info
   ---------------
   code id:   1000
   atrap:	  A372 (sysTrapHwrDisplayAttributes)

   History
   -------
   03/02/01  dia  Created
 */

// ----------------------------------
// Interfaces...
// ----------------------------------

// Silly precompiled harders aren't right on my system--don't really
// need them anyway...
#define PILOT_PRECOMPILED_HEADERS_OFF


/* System-wide */
#include <Pilot.h>

/* Hack specific */
#include "GreenLightHack.h"

// ----------------------------------
// Local types...
// ----------------------------------

typedef Err (*HwrDisplayAttributesFn) (Boolean set, UChar attr, void* dataP);


// ----------------------------------
// From missing header files...
// ----------------------------------

// ----------------------------------
// Trap Handler...
// ----------------------------------


/* Function: MyHwrDisplayAttributes()
   ----------------------------------
   A non-public function that is used to get info about the display.  We patch
   it to catch people turning on the backlight.

   @param set	If true, this is used to set the attribute; otherwise it's a "get".
   @param attr	The attribute to get or set.
   @param dataP Pointer to location to read from / write to.

   @return		non-zero upon error.
 */

Err
MyHwrDisplayAttributes (Boolean set, UChar attr, void* dataP);
Err
MyHwrDisplayAttributes (Boolean set, UChar attr, void* dataP)
{
	HwrDisplayAttributesFn oldTrap = NULL;
	Err err;
	
	err = FtrGet (kCreatorCode, kHwrDisplayAttributesPatchID, (DWord*) &oldTrap);
	if (err) { return err; }
	
	if ((attr == 0x12) && (set)) {
		const char kPixPolMask = 0x01;
		char * const LPOLCF = (char *)0xFFFFFA21;

		if (*((Boolean*)dataP)) {
			// Enable if turning on...
			(*LPOLCF) |= kPixPolMask;
		} else {
			// Disable if turning off...
			(*LPOLCF) &= ~kPixPolMask;
		}
	}
	err = oldTrap (set, attr, dataP);
	
	return err;
}
